#ifndef FLY_DE_H
#define FLY_DE_H

typedef struct _XDisplay Display;
typedef unsigned long Window; 
#include <fly/flylist.h>

#ifdef FLY_ORG_NAME
#define FLY_ORG FLY_ORG_NAME
#else
#define FLY_ORG "rusbitech"
#endif

#define FLYWM_LOCKER_WINDOW_NAME "FLYWM_LOCKER_WINDOW"

//compatible with kdeworkspace.h
enum FlyShutdownConfirm {
    ShutdownConfirmDefault =-1,// Obey the user's confirmation setting.
    ShutdownConfirmNo      = 0,// Don't confirm, shutdown without asking.
    ShutdownConfirmYes     = 1 // Always confirm, ask even if the user turned it off.
 };
enum FlyShutdownType {
	ShutdownTypeUnknown =-4,
	ShutdownTypeCancel  =-3,
	ShutdownTypeLock    =-2,
	ShutdownTypeDefault =-1,// Select previous action or the default if it's the first time.
    ShutdownTypeNone    = 0,// Only log out
    ShutdownTypeReboot  = 1,// Log out and reboot the machine.
    ShutdownTypeHalt    = 2,// Log out and halt the machine.
    ShutdownTypeLogout  = 3 // Temporary brain damage. Don't use. Same as ShutdownTypeNone

};
enum FlyShutdownMode {
    ShutdownModeDefault    = -1,// Select previous mode or the default if it's the first time.
    ShutdownModeSchedule   = 0, // Schedule a shutdown (halt or reboot) for the time all active have exited.
    ShutdownModeTryNow     = 1,	// Shut down, if no sessions are active. Otherwise do nothing.
    ShutdownModeForceNow   = 2, // Force shutdown. Kill any possibly active sessions
    ShutdownModeInteractive= 3  //Pop up a dialog asking the user what to do if sessions are
};

Window FindToplevelWindowByName(Display *dpy,Window win,const char *winName);

const char *GetOrganization();

//get message by index
char * GetFlyWMMessage(int index=-1,const char *defaultMsg="");

bool GetDMSessionsList(List<char> *lst); //don't forget to free by lst.Clear(true,true);

// WM and DM command interface (don't forget to finish command line with '\n')
bool GetFlyWMFIFOName(Display *,char *path);     //if path=NULL just check socket/fifo availability
bool GetFlyDMFIFOName(Display *,char *path);	 //if path=NULL just check socket/fifo availability
bool SendCommandToWM(Display *,const char *buf); //if buf=NULL or empty just check socket/fifo availability
bool SendCommandToDM(Display *,const char *buf); //if buf=NULL or empty just check socket/fifo availability

//this enum and function borrowed from Qt4
typedef enum  {
    DE_UNKNOWN,
    DE_KDE,
    DE_GNOME,
    DE_CDE,
    DE_4DWM, DE_FLY
} WhatDesktopEnvironment;
//in Qt4 use QX11Info::display(), QX11Info::appRootWindow() as arguments
WhatDesktopEnvironment whatDE(Display *dpy, Window rootWindow);

typedef enum {
    FLY_UNKNOWN,
    FLY_FAILSAFE,
    FLY_DESKTOP,
    FLY_TABLET,
    FLY_MINI,
    FLY_MOBILE,
    FLY_WESTON,
} WhatFlySession;

WhatFlySession whatFlySession(); //check $DESKTOP_SESSION

//get session name, just returns getenv("DESKTOP_SESSION")
const char * flySessionName();

//returns config suffix for most fly-wm config files, never returns NULL, delete [] it always
char * flySessionConfigSuffix(WhatFlySession stype=FLY_UNKNOWN);

//save session name in ~/.fly/last-desktop-session-name
bool flySessionSaveName(const char *nm=NULL);
//get session name from ~/.fly/last-desktop-session-name, never returns NULL, delete [] it always
char * flySessionGetSavedName();

void RenameDesktopShortcutCoord(const char *pathNameOld, char *newName,Display *dpy);

//save in desktop .directory the section: [IconPosition:fn] with X=x and Y=y fields
// to allow desktop item positioning by fly-wm
bool SaveDesktopItemPosition(const char *fn,const int x,const int y);
bool RemoveDesktopItemPosition(const char *fn);

void CreateStartMenu(const char *menuDir, bool updateOnly=false);

#endif
