#ifndef FLY_DESKTOP_ENTRY
#define FLY_DESKTOP_ENTRY

#include <stdlib.h>

//#if defined(__linux__) || defined(_linux_) || defined(linux)
//#include <linux/limits.h> //for PATH_MAX, NAME_MAX
//#else
#include <limits.h>
//#endif
#include <fly/flylist.h>

//#include "flyicontheme.h"
//#define ICON_CONTEXT_IN_DTOP_ENTRY 1

#define SOME_WORD_SIZE             64

//#define X_FLY_COPY	"X-FLY-COPY"

//class represents desktop entry file (see /usr[/local]/share/applications/*.desktop)
class FlyDesktopActionPrivate;
class FlyDesktopAction
{
public:
    FlyDesktopAction(const char *shortName);
    ~FlyDesktopAction();

    const char* getShortName();
    void  setShortName(const char *nm);

    const char* getName() const;
    void  		setName(const char *nm);
    const char* getLocalName() const;
    void  		setLocalName(const char *nm);
    const char* name() const;

    const char* getComment() const;
    void  		setComment(const char *cm);
    const char* getLocalComment() const;
    void  		setLocalComment(const char *cm);
    const char* comment() const;

    const char* getIcon() const;
    void  		setIcon(const char *e);
	const char* getLocalIcon() const;
	void  		setLocalIcon(const char *e);
	const char*	getIconContext() const;
	void  		setIconContext(const char *c);
	const char*	getLocalIconContext() const;
	void  		setLocalIconContext(const char *c);
	const char *icon() const; //get LocalIcon or Icon if no LocalIcon
	const char *iconContext() const; //get LocalIconContext or IconContext if no LocalIconContext

    const char* getPath() const;
    void  setPath(const char *p);

    const char* getExec() const;
    void  setExec(const char *e);

    void getOnlyShowIn(List<char> *osh) const;
    void setOnlyShowIn(const List<char> *osh);

	void getNotShowIn(List<char> *osh) const;
	void setNotShowIn(const List<char> *osh);

    FlyDesktopAction& operator=(const FlyDesktopAction&);
    bool  operator==(const FlyDesktopAction&) const;

private:
    FlyDesktopActionPrivate * p;
};

class FlyDesktopEntryPrivate;
class FlyDesktopEntry 
{
public:
    FlyDesktopEntry(const char *fpath=NULL);
    ~FlyDesktopEntry();

    // set/get path to read()/save()
    const char* getFilePath() const;
    void        setFilePath(const char *fpath);

    bool  load(); //reload *.desktop file
    bool  save(/*int x=-1,int y=-1,*/ bool saveEmpty=false); //save, dx,dy - coords on desktop, saveEmpty - save all fields
	bool  savePositionOnDesktop(int dx,int dy);

    bool  isValid() const;
    bool  isLoaded()const;

    const char* getEncoding() const;
    void  setEncoding(const char *enc);

    const char* getName() const;
    void  		setName(const char *nm);
    const char* getLocalName() const;
    void  		setLocalName(const char *nm);
	const char* name() const;

    const char* getGenericName() const;
    void  		setGenericName(const char *nm);
    const char* getLocalGenericName() const;
    void  		setLocalGenericName(const char *nm);
	const char* genericName() const;

    //"Application", "Link", "MimeType", "Directory", "Service", "ServiceType"
    const char* getType() const;
    void  setType(const char *t);

    void getFilePattern(List<char> *patt) const;
    void setFilePattern(const List<char> *patt);

    const char* getTryExec() const;
    void  setTryExec(const char *tre);

    bool  getNoDisplay() const;
    void  setNoDisplay(bool n);
   
    const char* getComment() const;
    void  		setComment(const char *cm);
    const char* getLocalComment() const;
    void  		setLocalComment(const char *cm);
	const char* comment() const;

    const char* getExec() const;
    void  setExec(const char *e);

    void  getActions(List<FlyDesktopAction> *ac) const; //get action (deep copy)
    void  setActions(const List<FlyDesktopAction> *ac); //set action (deep copy)
    
	const char*	getIcon() const;
	void  		setIcon(const char *c);
	const char*	getLocalIcon() const;
	void  		setLocalIcon(const char *c);
	const char*	getIconContext() const;
	void  		setIconContext(const char *c);
	const char*	getLocalIconContext() const;
	void  		setLocalIconContext(const char *c);
	const char *icon() const; //get LocalIcon or Icon if no LocalIcon
	const char *iconContext() const; //get LocalIconContext or IconContext if no LocalIconContext

    bool  getHidden() const;
    void  setHidden(bool h);

    const char* getPath() const;
    void  setPath(const char *p);

    bool  getTerminal() const;
    void  setTerminal(bool t);

    bool  getStartupNotify() const;
    void  setStartupNotify(bool t);

    const char* getStartupWMClass() const;
    void  setStartupWMClass(const char *st);

    const char* getSwallowTitle() const;
    void  		setSwallowTitle(const char *st);
    const char* getLocalSwallowTitle() const;
    void  		setLocalSwallowTitle(const char *st);
    const char* swallowTitle() const;

    const char* getSwallowExec() const;
    void  setSwallowExec(const char *se);

    void getMimeType(List<char> *mmt) const;
    void setMimeType(const List<char> *mmt);
    void getPatterns(List<char> *ptt) const;
    void setPatterns(const List<char> *ptt);
    
    const char* getDefaultApp() const;
    void  setDefaultApp(const char *da);

    void getSortOrder(List<char> *so) const;
    void setSortOrder(const List<char> *so);

    const char* getURL() const;
    void  setURL(const char *u);

    void getCategories(List<char> *ca) const;
    void setCategories(const List<char> *ca);

    void getOnlyShowIn(List<char> *osh) const;
    void setOnlyShowIn(const List<char> *osh);

	void getNotShowIn(List<char> *osh) const;
	void setNotShowIn(const List<char> *osh);

    const char* getReserved() const;
    void  setReserved(const char *r);

	typedef enum { 	Suitable		=0, 
					ExeEmpty		=1, 
					ExeMalformed	=2, 
					ExeNotExists	=3, 
					ExeNotFound		=4, 
					TryExeNotExists	=7,
					TryExeNotFound	=8, 
					UrlEmpty=9 } SuitableCode;
	typedef enum {	NoDisplay			=-3,
					NotForFly			=-2,
					Hidden				=-1,
					Displayable			=0 } DisplayableCode;
static 	bool isExecSuitable   (const char *execField, SuitableCode *retCode=NULL);
static 	bool isTryExecSuitable(const char *execField, SuitableCode *retCode=NULL);
    bool isSuitable(SuitableCode *retCode=NULL) const;
	bool isDisplayable(DisplayableCode *retCode=NULL) const;



    void inheritFields(const FlyDesktopEntry *from,bool onlyNotEmpty=false);

	const char * getVersion() const;
	void setVersion(const char *ver );

    void getKeywords(List<char> *kws) const;
    void setKeywords(const List<char> *kws);
	void getLocalKeywords(List<char> *kws) const;
	void setLocalKeywords(const List<char> *kws);
	void keywords(List<char> *kws) const;

	//get/set unknown key-values pairs
	void getUnknowns(      List<char> *keys,      List<char> *vals) const;
	void setUnknowns(const List<char> *keys,const List<char> *vals);
	//dont forget delete [] returned value
	char *getUnknownValue(const char *key);

    bool  operator==(const FlyDesktopEntry&);

private:
    FlyDesktopEntryPrivate * p;
};

List<char> *getDesktopEntryTypes(); //dont free returned List of Types

//find substring 'IconContex=value', allocate 'ctx' by 'new char[]' and copy 'value' there
//but there is getIconContext() under mcbc5
//bool extractIconContextFromString(const char *field,char **ctx); //obsolete

// search *.desktop in  /usr[/local]/share/applications
// for MimeType field 'mime'
// dont forget to Clear(true,false) returned list
FlyDesktopEntry * GetDesktopEntryForMime(const char *mime, List<FlyDesktopEntry> *allList=NULL); 

//get full abs. path for desktop entry file name (appname.desktop)
//dont forget to delete [] returned string
char * GetDesktopEntryFullPath(const char *entry);

//get all mime type aliases, dont forget Clear(tru,true) for list
void GetMimeAliases(const char *mimeName, List<char> *list);

//get file path with assignmets for mime apps, this funcs don't check if files exists

// $XDG_DATA_HOME/applications/mimeapps.list or ~/.local/share/applications/mimeapps.list, no check if file exists
const char * GetMimeToAppsUserTablePath();
// /usr/share/applications/mimeapps.list, no check if file exists
const char * GetMimeToAppsSystemTablePath(); //OBSOLETE, use GetMimeToAppsSystemTablePaths(char *out);
//returns colon separated paths list to find mimeapps.list in order from user specific to system wide, 
// $XDG_DATA_HOME/applications or ~/.local/share/applications, 
// $XDG_DATA_DIRS/applictions or /usr/local/share/applications:/usr/share/applications
// no check if file exists
void GetMimeToAppsSystemTablePaths(char *pathsOut);

//get assigned desktop entry(s), dont forget to delete returned desktop entry and list (Clear(true,false))
FlyDesktopEntry * GetAssignedDesktopEntryForMime(const char *mime,List<FlyDesktopEntry> *allList=NULL); 
//get icon(s) from assigned desktop entry, dont forget to delete returned string and list (Clear(true,true))
char * GetAssignedIconForMime(const char *mime,List<char> *allList=NULL); 

//get list of all apps desktop entries paths
//dont forget to Clear(true,true) returned list
void GetAllAppDesktopEntries(List<char> *);

//get list of all desktop entries search paths
//dont forget to Clear(true,true) returned list
void GetAllAppDesktopEntriesPaths(List<char> *);

// find folders, which contains .directory entry with non-empty Category field
void GetAllCategorizedFolderList(List<char> *folderList);


//get list of all apps desktop entries
//dont forget to Clear(true,false) returned list
void GetAllAppDesktopEntries(List<FlyDesktopEntry> *);

//format exec string according to Desktop Entry spec
char * ExpandExecFormatString(
            const char *execFormat,     //format
            const char *filePath,       // %f
            const char *url,            // %u
            const char *fileNoPath,     // %n
            const char *path,           // %d
            const char *icon,           // %i
            const char *miniIcon,       // %m
            const char *comment,        // %c
            const char *dtopFileName,   // %k
            List<char> *filePathList,   // %F
            List<char> *fileNoPathList, // %N
            List<char> *pathList,       // %D
            List<char> *urlList,        // %U
            const char *dev,            // %v
            const char defSep=' ');     //default separator for all lists
#endif

