#ifndef FLY_FILE_UTILS_H
#define FLY_FILE_UTILS_H

#include <stdlib.h>

//#if defined(__linux__) || defined (_linux_) || defined(linux)
//#include <linux/limits.h> //for PATH_MAX, NAME_MAX
//#else
#include <limits.h>
//#endif
//since some moment ARG_MAX may be not defined. so try so or get rid of it at all
//#ifndef ARG_MAX
//#define ARG_MAX (sysconf(_SC_ARG_MAX))
//#endif

typedef struct _XDisplay Display;
typedef unsigned long Window; 

#include <fly/flylist.h>

#include <fly/flydeapi.h>

//#ifdef __cplusplus
//extern "C"
//{
//#endif

//some popular program names
#define XDG_OPEN_UTIL           "xdg-open" // src
#define XDG_PROP_UTIL           "xdg-prop" // src
#define XDG_EDIT_UTIL           "xdg-edit" // src

#define FLYFM_OPEN_MANAGER      "fly-open"
#define FLYFM_OPEN   			"fly-open --exec" // srcs
#define FLYFM_OPEN_WITH  		"fly-open --open" // srcs

#define FLYFM_PROP_MANAGER      "fly-props"
#define FLYFM_PROP    			"fly-props --show" // srcs

#define FLYFM_FILE_MANAGER      "fly-fm" // src

#define FLYFM_FOPS_MANAGER   	"fly-fops"
#define FLYFM_REMOVE 			"fly-fops --remove"	// srcs
#define FLYFM_MOVE   			"fly-fops --move"	// srcs dst
#define FLYFM_COPY   			"fly-fops --copy"	// srcs dst
#define FLYFM_MOVE_TO  			"fly-fops -s --move"	// srcs dst
#define FLYFM_COPY_TO 			"fly-fops -s --copy"	// srcs dst
#define FLYFM_TRASH  	 		"fly-fops --trash"	// srcs
#define FLYFM_TRASH_CLEAN 		"fly-fops --clean"
#define FLYFM_EMAIL 		    "fly-fops --email"

#define FLY_DTE_EDITOR	   		"fly-dtedit"

#define FLY_TERM            	"fly-term"
#define FLY_TERM_EXEC       	FLY_TERM" -e " 			//at the end - command to execute
#define FLY_TERM_HOLD_EXEC  	FLY_TERM" --noclose -e "//at the end - command to execute

#define XTERM                   "xterm"
#define XTERM_EXEC              XTERM" -e "       //at the end - command to execute
#define XTERM_HOLD_EXEC         XTERM" -hold -e " //at the end - command to execute

#ifndef CP_FILE_CMD
#if defined(__linux__) || defined (_linux_) || defined(linux)
#define CP_FILE_CMD "/bin/cp -f"
#else
#define CP_FILE_CMD "/bin/cp"
#endif
#endif

#ifndef CP_DIR_CMD
#define CP_DIR_CMD CP_FILE_CMD" -r"
#endif


#ifndef CP_FILE_PRESERVE_CMD
#if defined(__linux__) || defined (_linux_) || defined(linux)
#define CP_FILE_PRESERVE_CMD CP_FILE_CMD" -p"
#else
#define CP_FILE_PRESERVE_CMD CP_FILE_CMD
#endif
#endif

#ifndef CP_DIR_PRESERVE_CMD
#if defined(__linux__) || defined (_linux_) || defined(linux)
#define CP_DIR_PRESERVE_CMD CP_DIR_CMD" -p"
#else
#define CP_DIR_PRESERVE_CMD CP_DIR_CMD
#endif
#endif

#ifndef MV_CMD
#if defined(__linux__) || defined (_linux_) || defined(linux)
#define MV_CMD "/bin/mv -f"
#else
#define MV_CMD "/bin/mv"
#endif
#endif

#ifndef RM_FILE_CMD
#define RM_FILE_CMD "/bin/rm -f"
#endif

#ifndef RM_DIR_CMD
#define RM_DIR_CMD RM_FILE_CMD" -r"
#endif

#ifndef MKDIR_CMD
#define MKDIR_CMD "/bin/mkdir -p"
#endif

//somestuff for WM remote command
#define VAR_RUN_DIR "/var/run"
#define FLY_TMP_DIR "/tmp"
#define FLY_FIFO_PREFIX "fly-wm-"

#define FLYWM_UPDATE_VAL_CMD    "FLYWM_UPDATE_VAL"
#define FLYWM_RESTART_CMD       "FLYWM_RESTART"

//fly-wm shared path
#define FLYWM_SHARED_DIR     "/usr/share/fly-wm"
#define FLYWM_SOUNDS_DIR     "sounds"
#define FLYWM_IMAGES_DIR     "images"
#define FLYWM_KEYMAPS_DIR    "keymaps"
#define FLYWM_DECOR_DIR      "decorations"

// user paths
#define FLY_THEME_DIR           "theme"
#define FLY_CURRENT_THEME_NAME  "current.themerc"
#define FLY_DEFAULT_THEME_NAME  "default.themerc" 

#define FLY_CONTROL_PANEL_DIR   "controlpanel"
#define FLY_DESKTOP_DIR         "Desktop"
#define FLY_DESKTOPS_DIR        "Desktops"
#define FLY_TOOLBAR_DIR         "toolbar"
#define FLY_MY_DOCUMENTS_DIR    "Documents"
#define FLY_START_MENU_DIR      "startmenu"
#define FLY_USER_TMP_DIR        "tmp"

//app shared paths (usually inside /usr/share/fly/...) - TODO: get all from flybuild vars
#define FLY_SHARED_DIR             "/usr/share/fly"
//where used?
#define FLY_SHARED_DOCS_DIR        "/usr/share/doc/fly"

#ifdef FLY_INS_DOCS
#define FLY_SHARED_HTML_DOCS_DIR   FLY_INS_DOCS
#else
#define FLY_SHARED_HTML_DOCS_DIR   FLY_SHARED_DOCS_DIR"/qthelp"
#endif

#ifdef FLY_INS_TRANS
#define FLY_SHARED_TRANS_DIR       FLY_INS_TRANS
#else
#define FLY_SHARED_TRANS_DIR       FLY_SHARED_DIR"/translations"
#endif

#define DEFAULT_APP_ICON            "system-run"
#define DEFAULT_APP_ICON_CTX        "Actions"



const char *GetFlyDir(FlyDir d); //returns pointers to static string like "~/.fly[/...]"
void 		ResetFlyDirs();		 //reset all cached dir paths

// find files by pattern in dir with possible traverse in subfolders (depth>0)
// only limited pattern accepted ("*name" or "name*")
bool GetFileList(const char *dir, int depthLevel, const char *pattern, List<char> *fileList, bool filesOnly=false);

//list of 'f' subdirs in XDG standard config dirs (dont forget clear filled list by Clear(true,true))
bool xdgConfigDirs(const char *f, List<char> *);
//lookup "f" in XDG standard config dirs and copy found path to path_to_fill (caller allocated or NULL)
bool findInXDGConfigDirs(const char *f, char *path_to_fill);

//lookup 1st existed "autostart" dir in XDG standard dirs and copy found path to path_to_fill (caller allocated or NULL)
bool autostartDir(char *path_to_fill);
//lookup all existed "autostart" dirs in XDG standard dirs (dont forget clear filled list by Clear(true,true))
bool autostartDirs(List<char> *pathList);
//OBSOLETED: install/uninstall/check 'appname.desktop' in user's autostart dir
//int  installInAutostart(const char *appname, bool inst, Display *dpy=NULL); //'dpy' is optional to quickly inform fly-wm about changes in startmenu
//OBSOLETED:
//bool isInAutostart(const char *appname);
//NEW: check if 'appname.desktop' installed and enabled(valid) in user's/system autostart, pathFind - buffer to return full desktop entry path
bool autostartEnabled(const char *appname, char *pathFind=NULL);
//NEW: install and-or enable/disable 'appname.desktop' in user's/system autostart
bool enableAutostart(const char *appname, bool enable, char *newExec=NULL, Display *dpy=NULL, bool deleteIfDisabled=true);

//returns 0 - OK; -1,-2,-3,... < 0 - errors in args, folder doesnt exist, permission denied, etc
int putEntryToRecentsMenu(const char *dtopentry,const char *icon,Display *, bool progs_only=true);

//get PPid field from /proc/pid/status
pid_t getPPid(pid_t pid);

//get run directory(dirPathOnly) or full cmdline by pid from /proc/pid/cmdline
int getRunDir(char *buf_out, int buf_len, pid_t pid=0, bool dirPathOnly=true);

bool setXrdbParameterInFile(const char *filePath,
						const char *paramName,const char *sep, const char *paramValue,
						bool apply_by_xrdb=false);

//unix 'which' command C-implementation
bool FindFileInPATH(const char *file, char *fullpath);

//return pointer to file extension ( "abcd.ext" - pointer to "ext" )
//return NULL if no extension or its length > extensionMaxLendth
bool fileNameExtension(const char *fileNamePath, unsigned int extensionMaxLendth=4);

// returned codes
// 0 - executable depends on libX11, 
// -1- everything is OK, but no dependency
// 1 - can't open executable
// 2 - can't read file beginning
// 3 - file is elf-archive
// 4 - can't get elf file header
// 5 - wrong elf file magic number
// 6 - can't process program header
bool isXApplication(const char *fullPath, int *retCode=NULL);

char *getResolvedAbsPath( const char *path );
//even if it returns false, check resolved - it can contain some path
bool resolveSymLink(const char *path, char *resolved,unsigned int size,bool checkstat=false);
bool makeRelativeSymlink(const char *dst, const char *slink);
bool isPathBusy(const char *path);

//remove dir recursively
// dir - path to dir
// inside_only - delete inside dir but not dir itself
// keep_dotdirectory_file - if true don't delete .directory file in dir
bool rmdirRecursive(const char *dir, bool inside_only=false, bool keep_dotdirectory_file=false);
//check if directory empty, if dont_check_dodirectory=true ignore .directory file
bool isDirEmpty(const char *path,bool dont_check_dodirectory=false);

//make new path like /parentPath/baseName(i).ext, where 
// parentPath - destination file path
// baseName   - base name without extension
// ext        - extension with dot (".png",".desktop",etc), 
// newPath    - returned full file path
// delete_and_reuse_if_not_busy - flag delete and reuse existed file if it is not busy
// int        - returned calculated index appended to base name
int MakeNewNumberedPath(const char *parentPath,const char *baseName,const char *ext,
                         char *newPath, bool delete_and_reuse_if_not_busy=false);

//lock/unlock file, 'block' - block function or immediately return
//fid has to be the same between both lock/unlock!
bool   flyLockFile(int fd, bool lockRead, bool block);
bool flyUnlockFile(int fd);

//find wall paper by short name (i.e. without full path and-or extension)
// the most suitable for screen size
bool FindWallPaperByName(const char *name, char *path,int screenWidth,int screenHeight,
     const char *svgConvertorFormat="fly-svg2png -w %d -h %d -i \"%s\" -o \"%s\"");

//both are OBSOLETED
bool isFSDeviceMounted(const char *dev,const char *mPoint, bool *asReadOnly=NULL);
//check fstab: 0 - OK, 1 - can't open /etc/fstab, 
//             2 - device or mpoint not found in fstab, 3 - wrong args
int checkFSDeviceInFSTab(const char *dev,const char *mPoint,
                         char *dev_out=NULL,    char *mpoint_out=NULL, 
                         char *fstype_out=NULL, char *opts_out=NULL, bool *asReadOnly_out=NULL);
int mountUmountFSDevice(const char *dev,const char *mPoint,const char *opts, bool mount);

// put environment variables like ~, $HOME, etc in retruned buf (don't forget to delete returned path) 
char*       ExtractPathName(const char* name);
const char* GetNextDelim(const char* path, char* name, int maxSize);

#endif
