#ifndef FLY_ICON_THEME
#define FLY_ICON_THEME

typedef struct _XDisplay Display;

#include <stdlib.h>

//#if defined(__linux__) || defined(_linux_) || defined(linux)
//#include <linux/limits.h> //for PATH_MAX, NAME_MAX
//#else
#include <limits.h>
//#endif

#include <fly/flylist.h>

#define MAX_INHERITS_NUM          2
#define CONTEXT_SIZE             30

//helper class for FlyIconTheme class
class FlyIconThemeSubdirPrivate;
class FlyIconThemeSubdir
{
public:
    FlyIconThemeSubdir(const char *path);
    ~FlyIconThemeSubdir();

    // set/get standard icon theme dir attributes
    int   getSize() const;
    void  setSize(int sz);

    int   getMaxSize() const;
    void  setMaxSize(int msz);

    int   getMinSize() const;
    void  setMinSize(int msz);

    int   getThreshold() const;
    void  setThreshold(int t);

    const char* getContext() const;
    void        setContext(const char*);

    const char* getType() const;
    void        setType(const char*);

    const char*	getPath() const;
    void        setPath(const char*);

    // find full icon path by icon name (without extension!)
    bool findIcon(const char *themePath,const char *iconName,const int iconSize,
                  const char *iconContext,char *iconFileName);
    bool matchesSize(int size) const;

private:
    FlyIconThemeSubdirPrivate * p;
};

class FlyIconThemePrivate;
class FlyIconTheme
{
public:
    enum SearchIconNameType {
    SearchMainIcon    = 0x1, //search main icon only
    SearchGenericIcon = 0x2, //search generic icon only
    SearchAnyIcon     = SearchMainIcon | SearchGenericIcon //seach main and if nto found then generic icon
    };

    FlyIconTheme(const char *name=NULL,const char *parentName=NULL, bool forceLoad = false);
    ~FlyIconTheme();

    // create static object of current theme and return pointer to it
    // this method checks if icon theme was changed and create new one if so
    // it's not thread-safe
    static FlyIconTheme *current(bool forceLoad=false);

    // create static object of current theme and return pointer to it
    // theme created by this method is not updated during the execution of the program
    // it's thread-safe
    static FlyIconTheme *instance();

    const char* getPath() const;      //returns full path to index.theme(index.dektop) file
    void  setPath(const char *path);  //set     full path to index.theme(index.dektop) file

    const char* getFile() const;  //returns file name of theme (index.theme or index.dektop)
    bool  setFile(const char *f); //set file name of theme (index.theme or index.dektop)

    const char* getThemeName() const;       //get theme internal name
    void  setThemeName(const char *name);   //set theme internal name

    bool  load(); //reload index.theme file
    bool  save(); //save main attributes (see below) in index.theme(index.dektop) file
    bool  isLoaded() const; //was it loaded at least once or not

    // set/get standard icon theme attributes

    const char* getName() const;               //get theme name
    void  setName(const char *name);     //set theme name

    const char* getLocalName() const;          //get theme name in local language
    void  setLocalName(const char *name);//set theme name in local language
    
    const char* getComment() const;            //get theme description
    void  setComment(const char*);       //set theme description

    const char* getLocalComment() const;       //get theme description in local language
    void  setLocalComment(const char*);  //set theme description in local language


    bool  getHidden() const;             //true if theme is hidden (may be we sholdn't show it)
    void  setHidden(bool);

    const char* getExampleIcon() const;        //picture as theme example (usually icon name without extension)
    void  setExampleIcon(const char*);   //picture as theme example (usually icon name without extension)

    void  getInheritedNames(List<char> *inhNames) const; //get inherited themes names (deep copy)
    void  setInheritedNames(const List<char> *inhNames); //set inherited themes names (deep copy)

    void  getSubdirs(List<FlyIconThemeSubdir> *sd) const; //get subdirs (deep copy)
    void  setSubdirs(const List<FlyIconThemeSubdir> *sd); //set subdirs (deep copy)

    //main API for apps: ---------------------------------------------

    // get icon theme path and description file name
    bool findThemePath(const char *nameIn,bool isInternalThemeNameIn,
                       char *themePathOut,char *fileNameOut,char *mainsectionOut);

    // find full icon path by icon short name (without extension!), by size and by context
    // contextIn - may be NULL or "" (then guessing will be by getIconContextAndPath())
    // fullFilePathOut - has to be non NULL field provided by caller
    bool getIcon(const char *iconNameIn, int iconSizeIn,const char *contextIn,char *fullFilePathOut);

    // guess icon context by name and get full path if required
    // ctx_ret - return pointer to static char field
    // fullPath - provided by caller or NULL,
    bool getIconContextAndPath(const char *iconName,const int iconSize, char const **ctx_ret, char *fullPath=NULL);

    //get full icon  path for mime type name and size
    // xdg_mime_get_icon() used first
    bool getMimeIcon(const char *mimeNameIn, int iconSizeIn, char *fullFilePathOut);

    //get mime icon name only, with check for full path (default size 32)
    bool getMimeIconName(const char *mimeNameIn, char *nameOut, const int nameSize);

    //get main and-or generic icon name from shared-mime-info database, depends on shared-mime-info completely
    bool findMimeIconNameXDG(const char *mimeNameIn, char *nameOut, const int nameSize, SearchIconNameType searchType) const;

    //get main icon name from shared-mime-info database, depends on shared-mime-info completely
    bool getMimeIconNameXDG(const char *mimeNameIn, char *nameOut, const int nameSize) const;

    //get generic icon name from shared-mime-info database, depends on shared-mime-info completely
    bool getMimeGenericIconNameXDG(const char *mimeNameIn, char *nameOut, const int nameSize) const;

    //just replace '/' with '-', i.e. text/x-makefile -> text-x-makefile
    bool getMimeIconNameSimple(const char *mimeNameIn, char *nameOut, const int nameSize) const;

    // fullPath- has to be non NULL field provided by caller
    bool getURLIcon(const char *url,const int iconSize, char *fullPath);

    bool getURLIconName(const char *url,char *iconNameOut, char *contextOut);

    bool getURLIcon(const char    *url,
                    const int     iconSize,
                    char          *ctxOut,
                    char          *nameOut,
                    char          *fullPathOut);

    //if iconName is full file name or path - check it in predefined directories
    static bool CheckFullPath(const char *iconName,char *fullPath_ret,char const **ctx_ret=NULL);

    //get colon separated dirs to search themes and icons
    static void GetThemesSearchDirs(char *seachDirs);

    //fill possible themes list by pattern ("index.* or "cursors")
    static void GetThemesIndexFiles(int depth,const char *pattern,List<char> *themesList,bool filesOnly);

    //get name of current icon theme from ~/.fly/theme/current.themerc or some default
    static bool getCurrentThemeName(char *nm);

    static bool getThemesNames(List<char> *retNames);

    static bool  getCursorThemeNames(List<char> *retNames);
    static char *getCurrentCursorThemeName(Display *dpy=NULL); //get 'Xcursor.theme: ' from ~/.Xresources, delete [] it
    static bool  setCurrentCursorThemeName(const char *nm);    //set 'Xcursor.theme: ' in   ~/.Xresources

    //returns suitable emblem icon size to make overlay with main icon
    static int GetIconEmblemSize(int iconSize/*, unsigned int dpi=96*/);

    //return one of standard sizes to fit (less) itemSize
    static int GetStandardIconSize(int itemSize);

    //recommended icon size
    static int GetIconSizeForDPI(int dpi, int inc);

    // ---------------------------------------------------------------
private:
    FlyIconThemePrivate * p;
};

#endif

