/*
 * list.h
 *
 */

#ifndef _LIST_H_
#define _LIST_H_

//#include <assert.h>
//#define FLYASSERT assert
/*#ifdef _DEBUG
//for backtrace
#include <stdio.h>
#include <stdlib.h>
#include <execinfo.h>
#endif*/
#include <libio.h> //NULL

template <class T>
class List 
{
public:
  class Iterator;
  friend class Iterator;

protected:
  class Item 
  {
	  friend class List;
	  friend class Iterator;
	
	  private:
		T* obj;
		Item* prev;
		Item* next;
		
	  public:
		Item(T* item) : obj(item), prev(NULL), next(NULL) {}
  };

public:
  class Iterator 
  {
  protected:
    List<T>* list;
    Item* current;

  public:
    Iterator() : list(NULL), current(NULL) {}
    Iterator(List<T>* lst) : list(lst), current(NULL) {}
    T* GetHead() {
      current = list->head;
      if (current)	return current->obj;
      return NULL;
    }
    T* GetTail() {
      current = list->tail;
      if (current)	return current->obj;
      return NULL;
    }
    T* GetPrev() {
      //FLYASSERT(current);
      if (current) current = current->prev;
      if (current) return current->obj;
      return NULL;
    }
    T* GetNext() {
      //FLYASSERT(current);
      if (current) current = current->next;
      if (current) return current->obj;
      return NULL;
    }
    T* GetCurrent() {
      if (current)	return current->obj;
      return NULL;
    }
    void InsertBefore(T* obj) {
      //FLYASSERT(current);
      if (!current || !current->prev) list->InsertHead(obj);
      else {
		Item* li = new Item(obj);
		li->next = current;
		li->prev = current->prev;
		current->prev->next = li;
		current->prev = li;
		list->size++;
      }
    }
    void InsertAfter(T* obj) {
      //FLYASSERT(current);
      if (!current || !current->next) list->InsertTail(obj);
      else {
		Item* li = new Item(obj);
		li->prev = current;
		li->next = current->next;
		current->next->prev = li;
		current->next = li;
		list->size++;
      }
    }
    // remove current item and advance iterator next
    T* Remove() {
      //FLYASSERT(current);
      if (current)
      {
        Item* li = current;
        current = current->next;
        list->Remove(li);
      }
      if (current)	return current->obj;
      return NULL;
    }
  };

protected:
  Item* head;
  Item* tail;
  int size;

private:
  void Remove(Item* li) {
    if (li->prev)	li->prev->next = li->next;
    else			head = li->next;
    if (li->next)   li->next->prev = li->prev;
    else    		tail = li->prev;
    delete li;
    size--;
  }

public:
  List() : head(NULL), tail(NULL), size(0) {}
  ~List() { 
/*#ifdef _DEBUG
      if (GetSize()>0)
      {
        void *array[32]; //max deep - 32
        char **strings;
        size_t size, i;
        fprintf(stderr,"flylist: nonempty List deleted - memleak possible, use Clear(true,[true,false])!\n");
        fprintf(stderr,"Backtrace:\n");
        size = backtrace(array, 32);
        strings = backtrace_symbols(array, size);
        for (i = 0; i < size; i++)
            fprintf(stderr,"%d: %s\n", i, strings[i]);
        free(strings);
      }
#endif*/
      Clear(false,false); 
   }

  T* GetHead() const {
    if (head)	return head->obj;
    return NULL;
  }
  T* GetTail() const {
    if (tail)	return tail->obj;
    return NULL;
  }
  int GetSize() const { return size; }

  void InsertHead(T* obj) {
    Item* li = new Item(obj);
    if (head) {
      head->prev = li;
      li->next = head;
    }
    else tail = li;
    head = li;
    size++;
  }
  void InsertTail(T* obj) {
    Item* li = new Item(obj);
    if (tail) {
      tail->next = li;
      li->prev = tail;
    }
    else head = li;
    tail = li;
    size++;
  }
  bool Remove(T* obj) {
    Item* li = head;

    while (li) {
      if (li->obj == obj) {
		  Remove(li);
		  return true;
      }
      li = li->next;
    }
    return false;
  }
  T* RemoveHead() {
    //FLYASSERT(head);
    if (head)
    {
      T* obj = head->obj;
      Remove(head);
      return obj;
    }
    return NULL;
  }
  T* RemoveTail() {
    //FLYASSERT(tail);
    if (tail)
    {
      T* obj = tail->obj;
      Remove(tail);
      return obj;
    }
    return NULL;
  }
  int Find(T *obj) const {
    Item* li;
    int i=0;
    for (li = head; li; li = li->next, i++) if (li->obj == obj) break;    
    if (li) return i;
    return -1;
  }
  T* Find(int i) const {
    Item* li;
    int j=0;
    for (li = head; li; li = li->next, j++) if (i==j) return li->obj;
    return NULL;
  }
  void Clear(bool delObj,bool delArray)
  {
      while (size>0) 
      {
        T* objToDel=(delObj && tail && tail->obj)?tail->obj:NULL;
        RemoveTail();
        if (objToDel) 
        {        
          if (delArray) delete [] objToDel;
          else          delete    objToDel;
        }
      }
  }
  bool isEmpty() { return GetSize()?false:true; }
};

extern int ListToString(  const List<char> *l,const char *sep,char *str,int str_sz,bool empty_too=true);
extern int ListFromString(List<char> *l,const char *sep,const char *str,           bool empty_too=true);
extern int ListFindString(const List<char> *l,const char *str,                     bool case_sens=true);
extern int ListStringSize(const List<char> *l,const char *sep);
extern int CopyStringList(const List<char> *from, List<char> *to);
extern bool CompareStringList(const List<char> *a, const List<char> *b);

#endif // _LIST_H_

