#ifndef FLY_MISC_H
#define FLY_MISC_H

#include <limits.h>

typedef struct _XDisplay Display;
typedef unsigned long Window; 
#include <fly/flylist.h>

//mem total in kiloBytes and cpu in Mhz
unsigned int GetMemTotal();
unsigned int GetCPUMhz();
bool         lightPlatform();

//get system locale string, for example "ru_RU.KOI8-R"
const char *GetSystemLocale();
//get 2 symbols language prefix "ru", "en", etc. (for example to get qt translation file suffix)
const char *GetLangPref();

int GetIPAddrList(List<char> * addrList, bool ipv4only=true);

int  GetDisplayNumberByName(char *display_name=NULL,Display *dpy=NULL);
bool isSessionRemote(char *dysplay_name=NULL,Display *dpy=NULL);
bool isSessionNested(char *dysplay_name=NULL,Display *dpy=NULL);

//find 'pattern' in 'ps -p uid' comand output
bool findInPS(const char *pattern);
//slow (who command used), to be replaced with fly-dmctl or direct utmp or ConsoleKit or systemd access
int  numWho(bool *diffUsers);

int findTouchscreenDevice(Display *, char *nameBuf=NULL, unsigned int nameBufSize=0, int *ids=NULL, unsigned int idsNum=0, bool all=false);
int findTouchscreenDevices(Display *,char *nameBuf=NULL, unsigned int nameBufSize=0, int *ids=NULL, unsigned int idsNum=0);

bool isAlsaActive(); //check "snd" in 'lsmod' output or /proc/asound/cards content
//bool isKernel26();

bool 			currentUser(char *buf_out, unsigned int buflen);
const char *	getUserName();
bool 			isUserRoot();
int 			getAllUserNames(char *buf, int bufsize,  uid_t lowUID=0);

//get wall paper size (dw,dh) and rate (ww,hh) for screen size (sw,sh)
void WallpaperSizeForScreen(int sw,int sh,int *dw,int *dh,int *ww,int *hh);

double GetXftDPI(Display *,int scr); //get current Xft dpi
void   SetXftDPI(Display *,int scr, double dpi, bool xrdb); //set xft dpi, if xrdb=true then set it in RESOURSE_MANAGER too

int GetFontSizeForDPI(int dpi, int inc);

//set 1st randr screen as primary
bool SetRRPrimary(Display *, Window rootw, int randr_major=0, int randr_minor=0);
//get physical dpi of primary screen, but wrond EDID or nested X servers report 0mm x 0mm
bool GetRRPrimaryDPI(Display *,Window rootw, double *xdpi, double *ydpi);

//find real dpi via randr, x root window, xft
double FindDPI(Display *,int screen,Window root);

/*-------------------------------------------------------------*/

/* Result codes returned after parsing a command line. */
#define kMavNoErr					0
#define kMavErrUnbalancedQuotes		1
#define kMavErrTooManyQuotePairs	2
#define kMavErrTooManyReDirections	3
#define kMavErrNoReDirectedFileName	4
#define kMavErrNoPipeCommand		5
#define kMavErrNoReDirectedInput	6
#define kMavErrBothPipeAndReDirOut	7

/* The different sets of quotes supported. */
#define ISQUOTE(c) (((c) == '"') || ((c) == '\'') || ((c) == '`'))

/* Max number of sets of nested quotes supported. */
#define kMaxQuotePairs 15

/* Special token types. */
#define kRegularArg 0
#define kReDirectOutArg 1
#define kReDirectOutAppendArg 2

/* Limit on number of arguments to track. */
#define kMaxArgs 128

typedef struct CmdLineInfo {
	int argCount;
	int isAppend;
	char outFileName[PATH_MAX];
	char pipeCmdLine[PATH_MAX];
	int savedStdout;
	int outFile;
	char argBuf[PATH_MAX*4/*ARG_MAX*/];  //ARG_MAX is 130000 - too big memory expenses!
	char *argVector[kMaxArgs + 4];
	int err;
	const char *errStr;
} CmdLineInfo, *CmdLineInfoPtr;

//#define NEWCMDLINEINFOPTR ((CmdLineInfoPtr) calloc((size_t)1, sizeof(CmdLineInfo)))

/* We put a few more things in the arg vector. */
#define CMDLINEFROMARGS(c,v) (v[(c) + 1])
#define CLIPFROMARGS(c,v) ((CmdLineInfoPtr) (v[(c) + 2]))

int MakeArgVector(char *str, CmdLineInfoPtr clp);
void ExpandDollarVariables(char *cmdLine, size_t sz);
/*-------------------------------------------------------------*/

#endif
