#ifndef FLY_SOUND_THEME
#define FLY_SOUND_THEME

#include <stdlib.h>

//#if defined(__linux__) || defined(_linux_) || defined(linux)
//#include <linux/limits.h> //for PATH_MAX, NAME_MAX
//#else
#include <limits.h>
//#endif

#include <fly/flylist.h>
#include <fly/flyicontheme.h>

#define SOUND_PROFILE_SIZE             30

//helper class for FlySoundTheme class
class FlySoundThemeSubdirPrivate;
class FlySoundThemeSubdir
{
public:
    FlySoundThemeSubdir(const char *path);
    ~FlySoundThemeSubdir();
    // set/get standard sound theme dir attributes
    const char* getContext() const;
    void        setContext(const char*);

	const char* getProfile() const;
    void        setProfile(const char*);

    const char*	getPath() const;
    void        setPath(const char*);

    // find full sound path by name (without extension!), locale, profile
    bool findSound(const char *themePath,const char *nm, 
				   const char *locale,   const char *prof,
				   char *soundFilePath) const;
private:
    FlySoundThemeSubdirPrivate * p;
};

class FlySoundThemePrivate;
class FlySoundTheme
{
public:
    FlySoundTheme(const char *name=NULL,const char *parentName=NULL);
    ~FlySoundTheme();

    const char* getPath() const;      //returns full path to index.theme file
    void  setPath(const char *path);  //set     full path to index.theme file

    const char* getFile() const;  //returns file name of theme (index.theme or index.dektop)
    bool  setFile(const char *f); //set file name of theme (index.theme or index.dektop)

    const char* getThemeName() const;     //get theme internal name
    void  setThemeName(const char *name); //set theme internal name

    bool  load(); 		//reload index.theme file
    bool  save(); 		//save main attributes (see below) in index.theme file
    bool  isLoaded() const; //was it loaded at least once or not

    // set/get standard sound theme attributes

    const char* getName() const;          //get theme name
    void  setName(const char *name);     //set theme name

    const char* getLocalName() const;    //get theme name in local language
    void  setLocalName(const char *name);//set theme name in local language
    
    const char* getComment() const;      //get theme description
    void  setComment(const char*);       //set theme description

    const char* getLocalComment() const; //get theme description in local language
    void  setLocalComment(const char*);  //set theme description in local language


    bool  getHidden() const;             //true if theme is hidden (may be we sholdn't show it)
    void  setHidden(bool);          

    const char* getExampleSound() const;  //sound as theme example (usually sound name without extension)
    void  setExampleSound(const char*);   //sound as theme example (usually sound name without extension)

    void  getInheritedNames(List<char> *inhNames) const; //get inherited themes names (deep copy)
    void  setInheritedNames(const List<char> *inhNames); //set inherited themes names (deep copy)

    void  getSubdirs(List<FlySoundThemeSubdir> *sd) const; //get subdirs (deep copy)
    void  setSubdirs(const List<FlySoundThemeSubdir> *sd); //set subdirs (deep copy)

    //main API for apps: ---------------------------------------------

    // get sound theme path and description file name 
    bool findThemePath(const char *nameIn,bool isInternalThemeNameIn,
                       char *themePathOut,char *fileNameOut,char *mainsectionOut);
        
    // find full sound path by sound short name (without extension!), by locale and by profile
    bool getSound(const char *soundNameIn, const char *localeIn, const char *profileIn,char *fullFilePathOut);

//get colon separated dirs to search themes and sounds
static void GetThemesSearchDirs(char *seachDirs);

//fill possible themes list by pattern ("index.* or "cursors")
static void GetThemesIndexFiles(int depth,const char *pattern,List<char> *themesList,bool filesOnly);

//get name of current sound theme from ~/.fly/theme/current.themerc or some default    
static bool getCurrentThemeName(char *nm);
//create static object of current theme and return pointer to it
//forceCheck - check if sound theme was changed and create new one if so
static FlySoundTheme *createCurrentTheme(bool forceCheck=false, bool forceLoad=false); 

// profile and locale - may be NULL or ""
// fullPath- has to be non NULL field provided by caller
static bool getSoundFromCurrentTheme(const char *soundName,const char *locale,const char *profile, char *fullPath);

static bool getThemesNames(List<char> *retNames);

    // ---------------------------------------------------------------

private:
    FlySoundThemePrivate * p;
};

#endif

