#ifndef FLY_TRASH
#define FLY_TRASH

class FlyTrash
{
public:
	//register (not move file itself) file in trash
	//args: 
    //	srcPath - full file/dir path to register in trash
	//	dstPath - destination path to move file/dir to trash (i.e. ~/.local/share/Trash/files/name[_i])
    static bool registerFile(const char *srcPath, char *dstPath); 

	//unregister file in trash
	//args:
    //	fileName - file/dir base name in trash dir (without path) or full path including trash path
    static bool unregisterFile(const char *fileName);

	//get file/dir path to restore from trash
	//args:
	//	fileNameOrPath - full file/dir path i.e. (GetFlyDir(USER_TRASH_DIR)/... or just file/dir name in trash dir without any '/'
	//  dstPath 	   - destination path to move file/dir from trash
    static bool getFilePathToRestore(const char *fileNameOrPath, char *dstPath);

	//get deletion date/time for file/dir in trash
	//args:
	//	fileNameOrPath - full file/dir path i.e. (GetFlyDir(USER_TRASH_DIR)/... or just file/dir name in trash dir without any '/'
	//  dateBuf     - buffer for date
	//  dateBufSize - buffer size
	static time_t getDeletionDate(const char *fileNameOrPath);

	static bool isFileInTrash(const char * path);

    static bool cleanRegistry();//clean trash registry dir
    static bool cleanFiles();	//clean trash files dir
    static bool cleanAll();		//clean both above

	//get trash top dir path (i.e. ~/.local/share/Trash/)
	static bool getTopDirPath(char *path);
	//get trash registry dir (i.e. ~/.local/share/Trash/info)
    static bool getInfoDirPath(char *path);
	//get trash file dir (i.e. ~/.local/share/Trash/files)
	static bool getFileDirPath(char *path);

	//check if trash empty (check files and info directories)
    static bool isEmpty();

	//NEW: real trash/untrash operations - not recommended due to
	// 		1)no progress indication 
	// 		2) work inside the same mounted device only
	//  filePath - full path to file/dir to move to trash
	static bool moveToTrash(const char *filePath); 
	//	fileNameOrPath - full file/dir path i.e. (GetFlyDir(USER_TRASH_DIR)/... or just file/dir name in trash dir without any '/'
	static bool restoreFromTrash(const char *fileNameOrPath);

private:
    static bool setPath(const char *file,const char *path,char *dstPath);
    static bool delPath(const char *fileName);
    static bool getPath(const char *fileName, char *pathOut);
	static bool getDate(const char *fileName, char *dateOut, int dateOutSize);

};
#endif
